/*
 *  pv_main.h
 *  ProteinViewer
 *
 *  Created by Naoto Morikawa on 6/15/06.
 *  Copyright (C) 2006, 2007, 2008 Naoto Morikawa. All rights reserved.
 *
 *  2006-10-01 version 0.1 release
 *  2006-11-20 version 0.2 release
 *  2007-04-09 support CO, CB, CG, CU, and CV in PE_ATOM_NAME
 *  2007-04-15 version 0.3 release
 */

#define DEBUG 1
#define STDOUT stdout
#define STDERR stdout
#define SUCCESS 0
#define FAIL 1
#define FLOAT_NEQ(fdata) ((fdata) * (fdata) >  1.0 / 1000.0)

#define PI 3.14159265358979323846  /* ==> math.h */
#define LineWidth 256	/* 80 for PDB */
/*
 Window coor (x,y)
 (0,0)   ->    (640,0)
   |
  \/
 (0,480) ->  (640,480)
*/
#define WindowWidth 640
#define WindowHeight 480
#define ON 1
#define OFF -1
#define CYLINDER_TYPE 0
#define SOLID_TYPE 1
#define WIRE_TYPE 2

#define PROTEIN_ONLY -1
#define BOTH_PRT_AND_MDL 0
#define MODEL_ONLY 1

#define SHOW_COOR_AXES 0

#define XYZW 0
#define XZYW 1
#define YXZW 2
#define YZXW 3
#define ZXYW 4
#define ZYXW 5
#define WXYZ 6
#define YWXZ 7
#define WYXZ 8
#define XWYZ 9
#define XYWZ 10
#define YXWZ 11
#define ZWXY 12
#define WXZY 13
#define XZWY 14
#define ZXWY 15
#define WZXY 16
#define XWZY 17
#define YZWX 18
#define ZYWX 19
#define ZWYX 20
#define WYZX 21
#define YWZX 22
#define WZYX 23

typedef enum pv_boolean {
	PV_TRUE,
	PV_FALSE
} PV_BOOLEAN;

/* Backbone atoms of an amino acid */
typedef enum pe_atom_name {
	N_NN,		/* the nitrogen atom of the amino group */
	N_CA,		/* the central carbon atom */
	N_CC,		/* the carbon atom of the carboxyl group */
	N_RNA,		/* RNA or DNA */
	N_CR,		/* the central carbon atom large red */
	N_CY,		/* the central carbon atom large yellow */
	N_CO,		/* the central carbon atom large orange *//* NM 20007-04-09 */
	N_CB,		/* the central carbon atom large blue *//* NM 20007-04-09 */
	N_CG,		/* the central carbon atom large green *//* NM 20007-04-09 */
	N_CU,		/* the central carbon atom small red *//* NM 20007-04-09 */
	N_CV,		/* the central carbon atom small orange *//* NM 20007-04-09 */
	N_CW,		/* the central carbon atom small blue */
	N_UNKNOWN	/* unknown */
} PE_ATOM_NAME;

typedef enum pe_code {
	C_UNKNOWN,
	C_UP,
	C_DOWN
} PE_CODE;

struct pv_settings {
	int StickBondOn;	/* scene() */
	int SolidTetraOn;	/* myinit() */
};

struct pe_vector {
	float x;
	float y;
	float z;
	float length;
};

/**
* struct pe_matrix - is a 3 by 3 matrix
 * @aij  : the (i,j) element
 *
 * a11 a12 a13
 * a21 a22 a23
 * a31 a32 a33
 */
struct pe_matrix {
	float a11;
	float a12;
	float a13;
	float a21;
	float a22;
	float a23;
	float a31;
	float a32;
	float a33;
};

struct pe_coor_system {
	struct pe_vector axes[4];         /* x, y, z, w */
	struct pe_vector origin;
	float ave_dis;
};

struct pe_slt_tile {
	float cx;		/* center of the tile */
	float cy;		/* center of the tile */
	float cz;		/* center of the tile */
	float tx;		/* top vertex of the tile */
	float ty;		/* top vertex of the tile */
	float tz;		/* top vertex of the tile */
	float bx;		/* bottom vertex of the tile */
	float by;		/* bottom vertex of the tile */
	float bz;		/* bottom vertex of the tile */
	float dx;		/* direction to the target */
	float dy;		/* direction to the target */
	float dz;		/* direction to the target */
	float error;            /*  |tile - nearest_atom| */
	int error_atom;
	int atom_flag;
	int top[4];              /* top vertex in R**4 */
	int type;                /*  XYZW, XZYW, ... */
	PE_CODE code;            /* C_UP or C_DOWN */
	int ter;                 /* end of a sequence */
	/* Only for protein viewer */
	/*  0   4   8  12
	 *  1   5   9  13
	 *  2   6  10  14
	 *  3   7  11  15
	 */
	float rot_mat[16];
};

struct pe_trn_table {
	char str[5];                 /* [xyz], [xzy], ... */
	/* top = ( 0, 0, 0 ) */
	float cx;                  /* center */
	float cy;                  /* center */
	float cz;                  /* center */
	float bx;                  /* bottom */
	float by;                  /* bottom */
	float bz;                  /* bottom */
	
	/* downward */
	float downward_cx;         /* center */
	float downward_cy;
	float downward_cz;
	float downward_U_next_cx;    /* center of the next tile */
	float downward_U_next_cy;
	float downward_U_next_cz;
	float downward_U_tx;         /* top vertex */
	float downward_U_ty;
	float downward_U_tz;
	float downward_U_bx;	/* bottom vertex */
	float downward_U_by;
	float downward_U_bz;
	int downward_U_top[4];        /* top vertex in R**4 */
	int downward_U_type;            /*  XYZW, XZYW, ...  */

	float downward_D_next_cx;    /* center of the next tile */
	float downward_D_next_cy;
	float downward_D_next_cz;
	float downward_D_tx;    /* top vertex */
	float downward_D_ty;
	float downward_D_tz;
	float downward_D_bx;	/* bottom vertex */
	float downward_D_by;
	float downward_D_bz;
	int downward_D_top[4];        /* top vertex in R**4 */
	int downward_D_type;            /*  XYZW, XZYW, ...  */

	/* upward */
	float upward_cx;        /* center */
	float upward_cy;
	float upward_cz;
	float upward_U_next_cx;    /* center of the next tile */
	float upward_U_next_cy;
	float upward_U_next_cz;
	float upward_U_tx;    /* top vertex */
	float upward_U_ty;
	float upward_U_tz;
	float upward_U_bx;	/* bottom vertex */
	float upward_U_by;
	float upward_U_bz;
	int upward_U_top[4];        /* top vertex in R**4 */
	int upward_U_type;            /*  XYZW, XZYW, ...  */

	float upward_D_next_cx;    /* center of the next tile */
	float upward_D_next_cy;
	float upward_D_next_cz;
	float upward_D_tx;    /* top vertex */
	float upward_D_ty;
	float upward_D_tz;
	float upward_D_bx;	/* bottom vertex */
	float upward_D_by;
	float upward_D_bz;
	int upward_D_top[4];        /* top vertex in R**4 */
	int upward_D_type;            /*  XYZW, XZYW, ...  */
};

/**
* struct pe_path_of_length_n - is an entry of an approximation error table
 * @code: pointer to an UP/DOWN sequence of length n (including the current one) 
 * @c   : the position of the center of the n-th tile
 * @bb_c: pointer to the position of all tiles (from 0-th to (n-1)-th)
 * @type: the type of the (n-1)-th tile
 */
struct pe_path_of_length_n {
	PE_CODE *code;
	struct pe_vector c;
	struct pe_vector * bb_c;
	int type;
};

struct pe_settings {
	float tiles_per_res;	/* standard #tile/#residues */
	int max_path;		/* 2**tipes_per_res */

	int along_backbone;
	PV_BOOLEAN rotate_tile;
};
